<?php

namespace Drupal\vidstack_player\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\Entity\MediaType;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\media\OEmbed\UrlResolverInterface;
use Drupal\media\Plugin\media\Source\OEmbedInterface;

/**
 * Plugin implementation of the Vidstack player remote video formatter.
 *
 * @FieldFormatter(
 *   id = "vidstack_player_remote_video",
 *   label = @Translation("Vidstack Player (Remote videos)"),
 *   field_types = {
 *     "link",
 *     "string",
 *     "string_long",
 *   }
 * )
 */
class VidstackPlayerRemoteVideoFormatter extends FormatterBase {

  use VidstackPlayerFormatterTrait;

  /**
   * The oEmbed url resolver service.
   *
   * @var \Drupal\media\OEmbed\UrlResolverInterface
   */
  protected $urlResolver;

  /**
   * Constructs an VidstackPlayerRemoteVideoFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin ID for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\media\OEmbed\UrlResolverInterface $url_resolver
   *   The oEmbed url resolver service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, UrlResolverInterface $url_resolver) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->urlResolver = $url_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('media.oembed.url_resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $loadStrategies = [
      'eager' => $this->t('Load immediately'),
      'idle' => $this->t('Load after page-load'),
      'visible' => $this->t('Load after visible'),
      'play' => $this->t('Load after play'),
    ];
    $summary = [];
    $summary[] = $this->t('Autoplay: <strong>@autoplay</strong><br>Loop: <strong>@loop</strong><br>Muted: <strong>@muted</strong><br>Default controls: <strong>@controls</strong><br>Load strategy: <strong>@load</strong>', [
      '@autoplay' => $this->getSetting('autoplay') ? $this->t('Yes') : $this->t('No'),
      '@loop' => $this->getSetting('loop') ? $this->t('Yes') : $this->t('No'),
      '@muted' => $this->getSetting('muted') ? $this->t('Yes') : $this->t('No'),
      '@controls' => $this->getSetting('controls') ? $this->t('Yes') : $this->t('No'),
      '@load' => $loadStrategies[$this->getSetting('load')],
    ]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return self::getSharedDefaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return $this->getSharedSettings($form);
  }

  /**
   * Analyzes the URL string to make sure that a valid provider is used.
   *
   * @param string $url
   *   The video URL string.
   *
   * @return bool
   *   Returns whether or not a valid provider is used.
   *
   * @throws \Drupal\media\OEmbed\ProviderException
   * @throws \Drupal\media\OEmbed\ResourceException
   */
  private function usesValidProvider(string $url): bool {
    $provider_name = NULL;
    if ($provider = $this->urlResolver->getProviderByUrl($url)) {
      $provider_name = strtolower($provider->getName());
    }
    return in_array($provider_name, ['vimeo', 'youtube']);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();
    $elements = [];
    foreach ($items as $delta => $item) {
      // Get the main property value (an url) and check if it uses a valid
      // provider:
      $main_property = $item->getFieldDefinition()->getFieldStorageDefinition()->getMainPropertyName();
      $value = $item->{$main_property};
      if (empty($value)) {
        continue;
      }
      if ($this->usesValidProvider($value)) {
        $elements[$delta] = [
          '#theme' => 'vidstack_player_remote_video',
          '#attributes' => [
            'class' => ['vidstack-player'],
            'src' => $value,
            'data-autoplay' => (bool) $settings['autoplay'],
            'data-loop' => (bool) $settings['loop'],
            'data-muted' => (bool) $settings['muted'],
            'data-controls' => (bool) $settings['controls'],
            'data-load' => $settings['load'],
          ],
        ];
      }
      else {
        $elements[$delta] = [
          '#type' => 'markup',
          '#markup' => $this->t('The video could not be loaded because @provider videos are not supported.', [
            '@provider' => $this->urlResolver->getProviderByUrl($value)->getName(),
          ]),
        ];
      }
    }
    if (!empty($elements)) {
      $elements['#attached']['library'][] = 'vidstack_player/init';
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    if ($field_definition->getTargetEntityTypeId() !== 'media') {
      return FALSE;
    }

    if (parent::isApplicable($field_definition)) {
      $media_type = $field_definition->getTargetBundle();

      if ($media_type) {
        $media_type = MediaType::load($media_type);
        return $media_type && $media_type->getSource() instanceof OEmbedInterface;
      }
    }
    return FALSE;
  }

}
