<?php

namespace Drupal\vidstack_player\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\file\Plugin\Field\FieldFormatter\FileMediaFormatterBase;
use Drupal\file\Plugin\Field\FieldFormatter\FileMediaFormatterInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the Vidstack player video formatter.
 *
 * @FieldFormatter(
 *   id = "vidstack_player_video",
 *   label = @Translation("Vidstack Player (Local videos)"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class VidstackPlayerVideoFormatter extends FileMediaFormatterBase implements FileMediaFormatterInterface {

  use VidstackPlayerFormatterTrait;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  private $entityFieldManager;

  /**
   * Constructs an VidstackPlayerRemoteVideoFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin ID for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityFieldManagerInterface $entityFieldManager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_field.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function getMediaType() {
    return 'video';
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $loadStrategies = [
      'eager' => $this->t('Load immediately'),
      'idle' => $this->t('Load after page-load'),
      'visible' => $this->t('Load after visible'),
      'play' => $this->t('Load after play'),
    ];
    $summary = [];
    $posterValue = '';
    switch ($this->getSetting('poster')) {
      case 'none':
        $posterValue = $this->t('None (Show first frame)');
        break;

      case 'custom':
        $posterValue = $this->t('Custom URL');
        break;

      default:
        $posterValue = $this->t('Field "@name" (@style)', [
          '@name' => $this->getSetting('poster'),
          '@style' => $this->getSetting('posterStyle') === '' ? $this->t('None (original image)') : image_style_options(TRUE)[$this->getSetting('posterStyle')],
        ]);
        break;
    }
    $summary[] = $this->t('Autoplay: <strong>@autoplay</strong><br>Loop: <strong>@loop</strong><br>Muted: <strong>@muted</strong><br>Default controls: <strong>@controls</strong><br>Load strategy: <strong>@load</strong><br>Poster image: <strong>@poster</strong><br>Poster load strategy: <strong>@posterLoad</strong>', [
      '@autoplay' => $this->getSetting('autoplay') ? $this->t('Yes') : $this->t('No'),
      '@loop' => $this->getSetting('loop') ? $this->t('Yes') : $this->t('No'),
      '@muted' => $this->getSetting('muted') ? $this->t('Yes') : $this->t('No'),
      '@controls' => $this->getSetting('controls') ? $this->t('Yes') : $this->t('No'),
      '@load' => $loadStrategies[$this->getSetting('load')],
      '@poster' => $posterValue,
      '@posterLoad' => $loadStrategies[$this->getSetting('posterLoad')],
    ]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return self::getSharedDefaultSettings() + [
      'poster' => 'none',
      'posterUrl' => '',
      'posterLoad' => 'visible',
      'posterStyle' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = $this->getSharedSettings($form);

    $form['poster'] = [
      '#title' => $this->t('Poster image'),
      '#type' => 'select',
      '#options' => [
        'none' => $this->t('None (Show first frame)'),
        'custom' => $this->t('Custom URL'),
        $this->t('Image fields from this entity:')->render() => $this->getEntityImageFields(),
      ],
      '#description' => $this->t('The image that will be displayed as a poster/thumbnail.'),
      '#default_value' => $this->getSetting('poster'),
    ];
    $form['posterUrl'] = [
      '#title' => $this->t('Custom poster image URL'),
      '#type' => 'textfield',
      '#description' => $this->t('The custom URL for the poster/thumbnail image.'),
      '#default_value' => $this->getSetting('posterUrl'),
      '#states' => [
        'visible' => [
          ':input[name="fields[field_media_video_file][settings_edit_form][settings][poster]"]' => ['value' => 'custom'],
        ],
      ],
    ];
    $form['posterStyle'] = [
      '#title' => $this->t('Poster image style'),
      '#type' => 'select',
      '#options' => image_style_options(FALSE),
      '#empty_option' => $this->t('None (original image)'),
      '#description' => $this->t('The image style to use for the poster image.'),
      '#default_value' => $this->getSetting('posterStyle'),
    ];
    $form['posterLoad'] = [
      '#title' => $this->t('Poster load strategy'),
      '#type' => 'select',
      '#options' => [
        'eager' => $this->t('Load immediately'),
        'idle' => $this->t('Load after the page is loaded'),
        'visible' => $this->t('Load after becoming visible'),
      ],
      '#description' => $this->t('The load strategy that will be used to determine when the poster/thumbnail should start loading.'),
      '#default_value' => $this->getSetting('posterLoad'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);
    $settings = $this->getSettings();

    foreach ($items as $delta => $item) {
      $elements[$delta][] = [
        '#theme' => 'vidstack_player_video',
      ];
      $posterUrl = '';
      switch ($settings['poster']) {
        case 'none':
          break;

        case 'custom':
          $posterUrl = $settings['posterUrl'];
          break;

        default:
          // Check if the poster is set to an image field on the entity, if not
          // break:
          if (!in_array($settings['poster'], array_keys($this->getEntityImageFields()), TRUE)) {
            break;
          }

          // Check if the poster field is empty (no referenced file). If not
          // break:
          $referenecedPosterFileId = $items->getEntity()->get($settings['poster'])->target_id;
          if ($referenecedPosterFileId === NULL) {
            break;
          }

          // We have a proper file id now, load the file and generate the file
          // uri based on the "posterStyle" setting:
          $posterFile = File::load($referenecedPosterFileId);
          if ($settings['posterStyle'] === '') {
            $posterUrl = $posterFile->createFileUrl(FALSE);
          }
          else {
            $posterFileUri = $posterFile->getFileUri();
            $posterUrl = ImageStyle::load($settings['posterStyle'])->buildUrl($posterFileUri);
          }
          break;
      }
      $elements[$delta]['#attributes']->addClass('vidstack-player')
        ->setAttribute('data-autoplay', (bool) $settings['autoplay'])
        ->setAttribute('data-loop', (bool) $settings['loop'])
        ->setAttribute('data-muted', (bool) $settings['muted'])
        ->setAttribute('data-controls', (bool) $settings['controls'])
        ->setAttribute('data-load', $settings['load'])
        ->setAttribute('data-poster', $posterUrl)
        ->setAttribute('data-poster-load', $settings['posterLoad']);
    }
    $elements['#attached']['library'][] = 'vidstack_player/init';
    return $elements;
  }

  /**
   * Helper function that gathers all image fields attached to the media entity.
   *
   * @return array
   *   The image fields of the media entity.
   */
  public function getEntityImageFields(): array {
    $imageFieldsArray = [];
    $imageFields = array_filter($this->entityFieldManager->getFieldDefinitions('media', 'video'),
    function (FieldDefinitionInterface $field) {
      return $field->getType() === 'image';
    });
    foreach ($imageFields as $imageField) {
      $imageFieldsArray[$imageField->getName()] = $imageField->getLabel() . ' (' . $imageField->getName() . ')';
    }
    return $imageFieldsArray;
  }

}
